@@ -2,10 +2,10 @@
 #'
 #' Test for Granger causality using out-of-sample prediction errors from an
 #' autoregression (AR) model, where some of the near-contemporaneous lags can be removed:
-#' \deqn{Y_t = \sum_{i=1}^p\alpha_iY_{t-i} + \sum_{i=lag.restrict+1}^p\beta_iX_{t-i} + e_t,}
+#' \deqn{Y_t = \sum_{i=1}^{p1}\alpha_iY_{t-i} + \sum_{i=lag.restrict+1}^{p2}\beta_iX_{t-i} + e_t,}
 #' where \eqn{Y_t} is the dependent variable,
 #' \eqn{X_t} is the cause variable,
-#' \eqn{p} is the AR order,
+#' \eqn{p1} and \eqn{p2} are the AR orders (if \code{p.free = FALSE}, \eqn{p1 = p2}),
 #' \eqn{lag.restrict} is the number of restricted first lags (see the argument \code{lag.restrict}).
 #'
 #' The tests include the MSE-t approach \insertCite{McCracken_2007}{funtimes} and
@@ -14,27 +14,23 @@
 #'
 #'
 #' @details
-#' Two versions of the bootstrap are used to derive the empirical distributions of the test statistics.
-#' In the original version, residuals of a restricted model under the null hypothesis of no Granger
-#' causality are bootstrapped to generate new data under the null. Then the full and restricted
+#' In the implemented bootstrapping, residuals of the restricted model under the null hypothesis of no Granger
+#' causality are bootstrapped to generate new data under the null hypothesis. Then, the full and restricted
 #' models are re-estimated on the bootstrapped data to obtain new (bootstrapped) forecast errors.
-#' In the fast bootstrap version, just the paired residuals from the full and restricted models are
-#' bootstrapped.
 #'
-#' In the current implementation, the bootstrapped \eqn{p}-value is calculated using equation 4.10 of
+#' In the current implementation, the bootstrapped \eqn{p}-value is calculated using Equation 4.10 in
 #' \insertCite{Davison_Hinkley_1997;textual}{funtimes}: \code{p.value} = (1 + \eqn{n}) / (\code{B} + 1),
 #' where \eqn{n} is the number of bootstrapped statistics smaller or equal to the observed statistic.
-#' In the fast bootstrap, \eqn{n} is the number of bootstrapped statistics greater or equal to 0.
 #'
-#' This function allows using different orders \eqn{p} when testing the Granger causation
-#' of \eqn{X} to \eqn{Y} and from \eqn{Y} to \eqn{X}
-#' (need to run the function twice, with different argument \code{cause}).
+#' This function tests the Granger causation
+#' of \eqn{X} to \eqn{Y} or from \eqn{Y} to \eqn{X}
+#' (to test in both directions, need to run the function twice, with different argument \code{cause}).
 #' To use the symmetric vector autoregression (VAR), use the function \code{\link{causality_predVAR}}.
 #'
 #' @param y matrix, data frame, or \code{ts} object with two columns
-#' (an explanatory and the dependent time-series variable). Missing values are not allowed.
+#' (a dependent and an explanatory time-series variable). Missing values are not allowed.
 #' @param cause name of the cause variable. If not specified, the first variable in
-#' \code{y} is treated as the cause and the second is treated as the dependent variable.
+#' \code{y} is treated as the dependent variable and the second is treated as the cause.
 #' @param p a vector of one or two positive integers specifying the order \eqn{p} of
 #' autoregressive dependence. The input of length one is recycled, then \code{p[1]} is used for
 #' the dependent variable and \code{p[2]} is used for the cause variable.
@@ -55,14 +51,14 @@
 #' The order is then selected using the Akaike information criterion (AIC; default),
 #' see the argument \code{k} to change the criterion.
 #' \code{lag.max} of length 2 automatically sets \code{p.free = TRUE}.
-#' @param k numeric specifying the weight of the equivalent degrees of freedom part
+#' @param k numeric scalar specifying the weight of the equivalent degrees of freedom part
 #' in the AIC formula. Default \code{k = 2} corresponds to the traditional AIC.
 #' Use \code{k = log(n)} to use the Bayesian information criterion instead
 #' (see \code{\link[stats]{extractAIC}}).
 #' @param B number of bootstrap replications. Default is 500.
 #' @param test a numeric value specifying the size of the testing set. If \code{test} < 1,
 #' the value is treated as a proportion of the sample size to be used as the testing set.
-#' Otherwise, \code{test} is rounded and \code{test} values are used as the testing set.
+#' Otherwise, \code{test} is treated as the number of the most recent values to be used as the testing set.
 #' Default is 0.3, which means that 30% of the sample is used for calculating
 #' out-of-sample errors. The testing set is always at the end of the time series.
 #' @param cl parameter to specify computer cluster for bootstrapping passed to
@@ -82,11 +78,10 @@
 #'   and stopped afterward (using \link[parallel]{stopCluster}).
 #' }
 #'
-#' @return Two lists (one for the fast bootstrap,
-#' another for the bootstrap under the null hypothesis) each containing the following elements:
-#' \item{result}{a table with the observed values of the test statistics and \eqn{p}-values.}
+#' @return A list containing the following elements:
+#' \item{stat}{a table with the observed values of the test statistics and \eqn{p}-values.}
 #' \item{cause}{the cause variable.}
-#' \item{p}{the AR order used.}
+#' \item{p}{the AR orders used for the dependent variable (\code{p[1]}) and for the cause variable (\code{p[2]}).}
 #'
 #' @references
 #' \insertAllCited{}
@@ -104,22 +99,27 @@
 #' causality_pred(Canada[,1:2], cause = "e", lag.max = 5, p.free = TRUE)
 #' causality_pred(Canada[,1:2], cause = "e", lag.restrict = 3, lag.max = 15, p.free = TRUE)
 #'
-#' # Example 2 (run in parallel, initiate the cluster manually):
+#' # Example 2 (run in parallel, initiate the cluster automatically)
 #' # Box & Jenkins time series
 #' # of sales and a leading indicator, see ?BJsales
 #'
+#' D <- cbind(BJsales.lead, BJsales)
+#' causality_pred(D, cause = "BJsales.lead", lag.max = 5, B = 1000, cl = NULL)
+#'
+#' # Example 3 (run in parallel, initiate the cluster manually)
+#'
 #' # Initiate a local cluster
 #' cores <- parallel::detectCores()
 #' cl <- parallel::makeCluster(cores)
 #' parallel::clusterSetRNGStream(cl, 123) # to make parallel computations reproducible
 #'
-#' D <- cbind(BJsales.lead, BJsales)
 #' causality_pred(D, cause = "BJsales.lead", lag.max = 5, B = 1000, cl = cl)
 #' causality_pred(D, cause = "BJsales.lead", lag.restrict = 3, p = 5, B = 1000, cl = cl)
 #' parallel::stopCluster(cl)
 #' }
 #'
-causality_pred <- function(y, cause = NULL,
+causality_pred <- function(y,
+                           cause = NULL,
                            p = NULL,
                            p.free = FALSE,
                            lag.restrict = 0L,
@@ -130,10 +130,10 @@ causality_pred <- function(y, cause = NULL,
                            cl = 1L)
 {
     if (!is.null(lag.max) && any(lag.max < 1L)) {
-        stop("lag.max must be positive integers.")
+        stop("lag.max must be positive integer(s).")
     }
     if (!is.null(p) && any(p < 1L)) {
-        stop("p must be positive integers.")
+        stop("p must be positive integer(s).")
     }
     bootparallel <- FALSE
     if (is.list(cl)) { #some other cluster supplied; use it but do not stop it
@@ -154,7 +154,7 @@ causality_pred <- function(y, cause = NULL,
     varnames <- colnames(y)
     if (length(varnames) != 2) stop("y must have 2 columns")
     if (is.null(cause)) {
-        cause <- varnames[1]
+        cause <- varnames[2]
     }
     dep <- setdiff(varnames, cause)[1] # dependent variable name
 
@@ -167,7 +167,7 @@ causality_pred <- function(y, cause = NULL,
     }
     n_test <- n - n_train
 
-    # Estimate p if using an information criterion
+    # Part 1: Estimate p on the training set if using an information criterion ----
     if (is.null(p) && is.null(lag.max)) {
         stop("Please specify p or lag.max.")
     }
@@ -175,21 +175,21 @@ causality_pred <- function(y, cause = NULL,
         if (length(lag.max) == 2) {
             p.free = TRUE
         } else {
-            lag.max <- c(lag.max, lag.max)
+            lag.max <- rep(lag.max[1], 2)
         }
         maxl <- max(lag.max)
         if (lag.restrict >= lag.max[2]) {
-            warning("lag.restrict >= lag.max. Using lag.restrict = 0 instead.")
+            warning("lag.restrict >= lag.max or lag.max[2]. Using lag.restrict = 0 instead.")
             lag.restrict <- 0
         }
+        # Embed both X and Y up to maxl to have the results of the same length
         if (lag.restrict > 0) {
-            lagX <- embed(y[,cause], maxl + 1)[, -c(1:(lag.restrict + 1)), drop = FALSE]
+            lagX <- embed(y[1:n_train, cause], maxl + 1)[, -c(1:(lag.restrict + 1)), drop = FALSE]
         } else {
-            lagX <- embed(y[,cause], maxl + 1)[, -1, drop = FALSE]
+            lagX <- embed(y[1:n_train, cause], maxl + 1)[, -1, drop = FALSE]
         }
-        lagY <- embed(y[,dep], maxl + 1)
+        lagY <- embed(y[1:n_train, dep], maxl + 1)
         # Information criterion for the model
-        # (use rows 1:(n_train - lag.max) to use training set only)
         if (p.free) {
             best.ic <- Inf
             for (p1 in 1:lag.max[1]) {
@@ -198,7 +198,7 @@ causality_pred <- function(y, cause = NULL,
                                                    lagY[, 2:(p1 + 1)],
                                                    lagX[, 1:(p2 - lag.restrict), drop = FALSE]),
                                          y = lagY[, 1])
-                    # see stats:::extractAIC.lm; but omit the scale option
+                    # see stats:::extractAIC.lm() but omit the scale option
                     nfit <- length(fit$residuals)
                     edf <- nfit - fit$df.residual
                     RSS <- sum(fit$residuals^2, na.rm = TRUE) # stats:::deviance.lm(fit)
@@ -229,13 +229,13 @@ causality_pred <- function(y, cause = NULL,
     if (length(p) == 2) {
         p.free = TRUE
     } else {
-        p <- c(p, p)
+        p <- rep(p[1], 2)
     }
     maxp <- max(p)
 
-    # Estimate model and get predictions on the testing set
+    # Part 2: Estimate model and get predictions on the testing set ----
     if (lag.restrict >= p[2]) {
-        warning("lag.restrict >= p. Using lag.restrict = 0 instead.")
+        warning("lag.restrict >= p or p[2]. Using lag.restrict = 0 instead.")
         lag.restrict <- 0
     }
     if (lag.restrict > 0) {
@@ -245,20 +245,17 @@ causality_pred <- function(y, cause = NULL,
     }
     lagY <- embed(y[,dep], maxp + 1)
     n_actual <- nrow(lagY)
-    # m_yx <- stats::lm(X1 ~ ., data = Dyx[1:(n_train - p),])
-    # m_yx <- stats::lm.fit(x = cbind(1, lagY[1:(n_train - p), -1], lagX[1:(n_train - p), ]),
-    #                     y = lagY[1:(n_train - p), 1])
 
     # Get 1-step ahead forecasts from the models (recursive)
     FCST <- sapply(1:n_test - 1, function(i) { # i = 0
         # estimate full and restricted models
         m_yx <- stats::lm.fit(x = cbind(1,
-                                        lagY[(1 + i):(n_train - maxp + i), 2:(p[1] + 1), drop = FALSE],
-                                        lagX[(1 + i):(n_train - maxp + i), 1:(p[2] - lag.restrict), drop = FALSE]),
-                              y = lagY[(1 + i):(n_train - maxp + i), 1, drop = FALSE])
+                                        lagY[1:(n_train - maxp + i), 2:(p[1] + 1), drop = FALSE],
+                                        lagX[1:(n_train - maxp + i), 1:(p[2] - lag.restrict), drop = FALSE]),
+                              y = lagY[1:(n_train - maxp + i), 1, drop = FALSE])
         m_y <- stats::lm.fit(x = cbind(1,
-                                       lagY[(1 + i):(n_train - maxp + i), 2:(p[1] + 1), drop = FALSE]),
-                             y = lagY[(1 + i):(n_train - maxp + i), 1, drop = FALSE])
+                                       lagY[1:(n_train - maxp + i), 2:(p[1] + 1), drop = FALSE]),
+                             y = lagY[1:(n_train - maxp + i), 1, drop = FALSE])
         # get predictions
         c(m_yx$coefficients %*% c(1, lagY[(n_train - maxp + i + 1), 2:(p[1] + 1)], lagX[(n_train - maxp + i + 1), 1:(p[2] - lag.restrict)]),
           m_y$coefficients %*% c(1, lagY[(n_train - maxp + i + 1), 2:(p[1] + 1)]))
@@ -269,27 +266,9 @@ causality_pred <- function(y, cause = NULL,
     # Observed test statistics
     OBS <- caustests(efull, eres) # caustests <- funtimes:::caustests
 
-    # Fast bootstrap (only the out-of-sample errors)
-    BOOT <- #parallel::parSapply(cl, X = 1:B, FUN = function(b) {
-        sapply(1:B, FUN = function(b) {
-            # bootstrap prediction errors
-            ind <- sample(n_test, replace = TRUE)
-            efullb <- efull[ind]
-            # ind <- sample(n_test, replace = TRUE)
-            eresb <- eres[ind]
-            # get bootstrapped statistics
-            caustests(efullb, eresb)
-        })
-    Fast <- list(result = data.frame(MSEt = c(OBS["MSEt"],
-                                              (sum(BOOT["MSEt",] >= 0) + 1) / (B + 1)),
-                                     MSEcor = c(OBS["MSEcor"],
-                                                (sum(BOOT["MSEcor",] >= 0) + 1) / (B + 1)),
-                                     row.names = c("stat_obs", "p_boot")),
-                 cause = cause,
-                 p = p)
-
-    # Bootstrap restricted model estimated on the full sample
-    m_y <- stats::lm.fit(x = lagY[, 2:(p[1] + 1), drop = FALSE],
+    # Bootstrap restricted model estimated on the full sample (Attanasio et al. 2016)
+    m_y <- stats::lm.fit(x = cbind(1,
+                                   lagY[, 2:(p[1] + 1), drop = FALSE]),
                          y = lagY[, 1, drop = FALSE])
     m_y_fit <- m_y$fitted.values
     m_y_res <- m_y$residuals
@@ -299,12 +278,12 @@ causality_pred <- function(y, cause = NULL,
             FCST <- sapply(1:n_test - 1, function(i) { # i = 0
                 # estimate full and restricted models
                 m_yx <- stats::lm.fit(x = cbind(1,
-                                                lagY[(1 + i):(n_train - maxp + i), 2:(p[1] + 1), drop = FALSE],
-                                                lagX[(1 + i):(n_train - maxp + i), 1:(p[2] - lag.restrict), drop = FALSE]),
-                                      y = dy_boot[(1 + i):(n_train - maxp + i)])
+                                                lagY[1:(n_train - maxp + i), 2:(p[1] + 1), drop = FALSE],
+                                                lagX[1:(n_train - maxp + i), 1:(p[2] - lag.restrict), drop = FALSE]),
+                                      y = dy_boot[1:(n_train - maxp + i)])
                 m_y <- stats::lm.fit(x = cbind(1,
-                                               lagY[(1 + i):(n_train - maxp + i), 2:(p[1] + 1), drop = FALSE]),
-                                     y = dy_boot[(1 + i):(n_train - maxp + i)])
+                                               lagY[1:(n_train - maxp + i), 2:(p[1] + 1), drop = FALSE]),
+                                     y = dy_boot[1:(n_train - maxp + i)])
                 # get predictions
                 c(m_yx$coefficients %*% c(1, lagY[(n_train - maxp + i + 1), 2:(p[1] + 1)], lagX[(n_train - maxp + i + 1), 1:(p[2] - lag.restrict)]),
                   m_y$coefficients %*% c(1, lagY[(n_train - maxp + i + 1), 2:(p[1] + 1)]))
@@ -319,35 +298,37 @@ causality_pred <- function(y, cause = NULL,
             parallel::stopCluster(cl)
         }
     } else {
-        BOOT0 <- #parallel::parSapply(cl, X = 1:B, FUN = function(b) {
-            sapply(1:B, FUN = function(b) {
-                dy_boot <- m_y_fit + sample(m_y_res, replace = TRUE)
-                FCST <- sapply(1:n_test - 1, function(i) { # i = 0
-                    # estimate full and restricted models
-                    m_yx <- stats::lm.fit(x = cbind(1,
-                                                    lagY[(1 + i):(n_train - maxp + i), 2:(p[1] + 1), drop = FALSE],
-                                                    lagX[(1 + i):(n_train - maxp + i), 1:(p[2] - lag.restrict), drop = FALSE]),
-                                          y = dy_boot[(1 + i):(n_train - maxp + i)])
-                    m_y <- stats::lm.fit(x = cbind(1,
-                                                   lagY[(1 + i):(n_train - maxp + i), 2:(p[1] + 1), drop = FALSE]),
-                                         y = dy_boot[(1 + i):(n_train - maxp + i)])
-                    # get predictions
-                    c(m_yx$coefficients %*% c(1, lagY[(n_train - maxp + i + 1), 2:(p[1] + 1)], lagX[(n_train - maxp + i + 1), 1:(p[2] - lag.restrict)]),
-                      m_y$coefficients %*% c(1, lagY[(n_train - maxp + i + 1), 2:(p[1] + 1)]))
-                })
-                # Forecast errors
-                efullb <- dy_boot[(n_train + 1):n - maxp] - FCST[1,]
-                eresb <- dy_boot[(n_train + 1):n - maxp] - FCST[2,]
-                # test statistics
-                caustests(efullb, eresb)
+        BOOT0 <- sapply(1:B, FUN = function(b) {
+            dy_boot <- m_y_fit + sample(m_y_res, replace = TRUE)
+            FCST <- sapply(1:n_test - 1, function(i) { # i = 0
+                # estimate full and restricted models
+                m_yx <- stats::lm.fit(x = cbind(1,
+                                                lagY[1:(n_train - maxp + i), 2:(p[1] + 1), drop = FALSE],
+                                                lagX[1:(n_train - maxp + i), 1:(p[2] - lag.restrict), drop = FALSE]),
+                                      y = dy_boot[1:(n_train - maxp + i)])
+                m_y <- stats::lm.fit(x = cbind(1,
+                                               lagY[1:(n_train - maxp + i), 2:(p[1] + 1), drop = FALSE]),
+                                     y = dy_boot[1:(n_train - maxp + i)])
+                # get predictions
+                c(m_yx$coefficients %*% c(1, lagY[(n_train - maxp + i + 1), 2:(p[1] + 1)], lagX[(n_train - maxp + i + 1), 1:(p[2] - lag.restrict)]),
+                  m_y$coefficients %*% c(1, lagY[(n_train - maxp + i + 1), 2:(p[1] + 1)]))
             })
+            # Forecast errors
+            efullb <- dy_boot[(n_train + 1):n - maxp] - FCST[1,]
+            eresb <- dy_boot[(n_train + 1):n - maxp] - FCST[2,]
+            # test statistics
+            caustests(efullb, eresb)
+        })
     } # end sequential bootstrap
-    FullH0 <- list(result = data.frame(MSEt = c(OBS["MSEt"],
-                                                (sum(BOOT0["MSEt",] <= OBS["MSEt"]) + 1) / (B + 1)),
-                                       MSEcor = c(OBS["MSEcor"],
-                                                  (sum(BOOT0["MSEcor",] <= OBS["MSEcor"]) + 1) / (B + 1)),
-                                       row.names = c("stat_obs", "p_boot")),
-                   cause = cause,
-                   p = p)
-    return(list(Fast = Fast, FullH0 = FullH0))
+    list(stat = data.frame(MSEt = c(OBS["MSEt"],
+                                    (sum(BOOT0["MSEt",] <= OBS["MSEt"]) + 1) / (B + 1)),
+                           MSEcor = c(OBS["MSEcor"],
+                                      (sum(BOOT0["MSEcor",] <= OBS["MSEcor"]) + 1) / (B + 1)),
+                           OOSF = c(OBS["OOSF"],
+                                    (sum(BOOT0["OOSF",] >= OBS["OOSF"]) + 1) / (B + 1)),
+                           EN = c(OBS["EN"],
+                                  (sum(BOOT0["EN",] >= OBS["EN"]) + 1) / (B + 1)),
+                           row.names = c("stat_obs", "p_boot")),
+         cause = cause,
+         p = p)
 }
