@@ -2,6 +2,10 @@
 knitr::opts_chunk$set(collapse = TRUE, comment = "#")
 library(funtimes)
 
+## ----eval = FALSE-------------------------------------------------------------
+#  install.packages("funtimes")
+#  library(funtimes)
+
 ## ----eval = FALSE-------------------------------------------------------------
 #  ?beales
 
@@ -14,8 +18,10 @@ B10 <- beales(x = discharge, y = loads)
 
 ## -----------------------------------------------------------------------------
 ls(B10)
-B10$se
+
+## -----------------------------------------------------------------------------
 B10$N
+B10$se
 
 ## -----------------------------------------------------------------------------
 B11 <- beales(x = discharge, y = loads, level = 0.9)
@@ -50,3 +56,6 @@ B32 <- beales(x = discharge2, y = loads2, N = 12, d = 15)
 ## -----------------------------------------------------------------------------
 B32$nhat
 
+## -----------------------------------------------------------------------------
+citation("funtimes")
+
